@@ -37,15 +37,14 @@ from qcodes.dataset.sqlite.queries import (
     get_guids_from_run_spec, get_last_experiment, get_metadata,
     get_metadata_from_run_id, get_parameter_data, get_parent_dataset_links,
     get_run_description, get_run_timestamp_from_run_id, get_runid_from_guid,
-    get_sample_name_from_experiment_id, get_setpoints, mark_run_complete,
+    get_sample_name_from_experiment_id, mark_run_complete,
     remove_trigger, run_exists, set_run_timestamp, update_parent_datasets,
     update_run_description)
 from qcodes.dataset.sqlite.query_helpers import (VALUE, VALUES,
                                                  insert_many_values,
-                                                 insert_values, length, one,
+                                                 length, one,
                                                  select_one_where)
 from qcodes.instrument.parameter import _BaseParameter
-from qcodes.utils.deprecate import deprecate
 
 from .data_set_cache import DataSetCache
 from .descriptions.versioning import serialization as serial
@@ -58,9 +57,6 @@ if TYPE_CHECKING:
 log = logging.getLogger(__name__)
 
 
-# TODO: as of now every time a result is inserted with add_result the db is
-# saved same for add_results. IS THIS THE BEHAVIOUR WE WANT?
-
 # TODO: storing parameters in separate table as an extension (dropping
 # the column parametenrs would be much nicer
 
@@ -282,15 +278,15 @@ class DataSet(Sized):
                          'captured_run_id', 'captured_counter')
     background_sleep_time = 1e-3
 
-    def __init__(self, path_to_db: str = None,
+    def __init__(self, path_to_db: Optional[str] = None,
                  run_id: Optional[int] = None,
                  conn: Optional[ConnectionPlus] = None,
                  exp_id: Optional[int] = None,
-                 name: str = None,
+                 name: Optional[str] = None,
                  specs: Optional[SpecsOrInterDeps] = None,
                  values: Optional[VALUES] = None,
                  metadata: Optional[Mapping[str, Any]] = None,
-                 shapes: Shapes = None) -> None:
+                 shapes: Optional[Shapes] = None) -> None:
         """
         Create a new :class:`.DataSet` object. The object can either hold a new run or
         an already existing run. If a ``run_id`` is provided, then an old run is
@@ -779,6 +775,8 @@ class DataSet(Sized):
         """
         Mark :class:`.DataSet` as complete and thus read only and notify the subscribers
         """
+        if self.completed:
+            return
         if self.pristine:
             raise RuntimeError('Can not mark DataSet as complete before it '
                                'has been marked as started.')
@@ -794,43 +792,6 @@ class DataSet(Sized):
             sub.done_callback()
         self._ensure_dataset_written()
 
-    @deprecate(alternative='add_results')
-    def add_result(self, results: Mapping[str, VALUE]) -> int:
-        """
-        Add a logically single result to existing parameters
-
-        Args:
-            results: dictionary with name of a parameter as the key and the
-                value to associate as the value.
-
-        Returns:
-            index in the DataSet that the result was stored at
-
-        If a parameter exist in the :class:`.DataSet` and it's not in the results
-        dictionary, "Null" values are inserted.
-
-        It is an error to provide a value for a key or keyword that is not
-        the name of a parameter in this :class:`.DataSet`.
-
-        It is an error to add results to a completed :class:`.DataSet`.
-        """
-
-        self._raise_if_not_writable()
-
-        try:
-            parameters = [self._rundescriber.interdeps._id_to_paramspec[name]
-                          for name in results]
-            self._rundescriber.interdeps.validate_subset(parameters)
-        except DependencyError as de:
-            raise ValueError(
-                'Can not add result, missing setpoint values') from de
-
-        index = insert_values(self.conn, self.table_name,
-                              list(results.keys()),
-                              list(results.values())
-                              )
-        return index
-
     def add_results(self, results: Sequence[Mapping[str, VALUE]]) -> None:
         """
         Adds a sequence of results to the :class:`.DataSet`.
@@ -871,24 +832,6 @@ class DataSet(Sized):
             raise CompletedError('This DataSet is complete, no further '
                                  'results can be added to it.')
 
-    @deprecate(alternative='add_results')
-    def add_result_to_queue(self,
-                            results: Sequence[Mapping[str, VALUE]]) -> None:
-        """
-        Add result to the output Queue from which a worker in a separate thread
-        consumes
-        """
-        self._raise_if_not_writable()
-
-        expected_keys = frozenset.union(*[frozenset(d) for d in results])
-        values = [[d.get(k, None) for k in expected_keys] for d in results]
-
-        item = {'keys': list(expected_keys), 'values': values,
-                "table_name": self.table_name}
-        writer_status = self._writer_status
-
-        writer_status.data_write_queue.put(item)
-
     def _ensure_dataset_written(self) -> None:
         writer_status = self._writer_status
 
@@ -897,14 +840,14 @@ class DataSet(Sized):
             while self.run_id in writer_status.active_datasets:
                 time.sleep(self.background_sleep_time)
         else:
-            writer_status.active_datasets.remove(self.run_id)
+            if self.run_id in writer_status.active_datasets:
+                writer_status.active_datasets.remove(self.run_id)
         if len(writer_status.active_datasets) == 0:
             writer_status.write_in_background = None
             if writer_status.bg_writer is not None:
                 writer_status.bg_writer.shutdown()
                 writer_status.bg_writer = None
 
-
     @staticmethod
     def _validate_parameters(*params: Union[str, ParamSpec, _BaseParameter]
                              ) -> List[str]:
@@ -942,10 +885,21 @@ class DataSet(Sized):
         The values are returned as a dictionary with names of the requested
         parameters as keys and values consisting of dictionaries with the
         names of the parameters and its dependencies as keys and numpy arrays
-        of the data as values. If some of the parameters are stored as arrays
+        of the data as values. If the dataset has a shape recorded
+        in its metadata and the number of datapoints recorded matches the
+        expected number of points the data will be returned as numpy arrays
+        in this shape. If there are less datapoints recorded than expected
+        from the metadata the dataset will be returned as is. This could happen
+        if you call `get_parameter_data` on an incomplete dataset. See
+        :py:meth:`dataset.cache.data <.DataSetCache.data>` for an implementation that
+        returns the data with the expected shape using `NaN` or zeros as
+        placeholders.
+
+        If there are more datapoints than expected the dataset will be returned
+        as is and a warning raised.
+
+        If some of the parameters are stored as arrays
         the remaining parameters are expanded to the same shape as these.
-        Apart from this expansion the data returned by this method
-        is the transpose of the date returned by ``get_data``.
 
         If provided, the start and end arguments select a range of results
         by result count (index). If the range is empty - that is, if the end is
@@ -1129,28 +1083,6 @@ class DataSet(Sized):
                 df_to_save = pd.concat(dfs_to_save, axis=1)
                 df_to_save.to_csv(path_or_buf=dst, header=False, sep='\t')
 
-    @deprecate(alternative="get_parameter_data")
-    def get_setpoints(self, param_name: str) -> Dict[str, List[List[Any]]]:
-        """
-        Get the setpoints for the specified parameter
-
-        Args:
-            param_name: The name of the parameter for which to get the
-                setpoints
-        """
-
-        paramspec: ParamSpecBase = self._rundescriber.interdeps._id_to_paramspec[param_name]
-
-        if param_name not in self.parameters:
-            raise ValueError('Unknown parameter, not in this DataSet')
-
-        if paramspec not in self._rundescriber.interdeps.dependencies.keys():
-            raise ValueError(f'Parameter {param_name} has no setpoints.')
-
-        setpoints = get_setpoints(self.conn, self.table_name, param_name)
-
-        return setpoints
-
     def subscribe(self,
                   callback: Callable[[Any, int, Optional[Any]], None],
                   min_wait: int = 0,
@@ -1247,7 +1179,7 @@ class DataSet(Sized):
 
         Before we can enqueue the results, all values of the results dict
         must have the same length. We enqueue each parameter tree seperately,
-        effectively mimicking making one call to add_result per parameter
+        effectively mimicking making one call to add_results per parameter
         tree.
 
         Deal with 'numeric' type parameters. If a 'numeric' top level parameter
