@@ -191,7 +191,7 @@ class InstrumentBase(Metadatable, DelegateAttributes):
         if params_to_skip_update is None:
             params_to_skip_update = []
 
-        snap = {
+        snap: Dict[str, Any] = {
             "functions": {name: func.snapshot(update=update)
                           for name, func in self.functions.items()},
             "submodules": {name: subm.snapshot(update=update)
@@ -207,7 +207,6 @@ class InstrumentBase(Metadatable, DelegateAttributes):
                 update_par: Optional[bool] = False
             else:
                 update_par = update
-
             try:
                 snap['parameters'][name] = param.snapshot(update=update_par)
             except:
@@ -472,7 +471,7 @@ class Instrument(InstrumentBase, AbstractInstrument):
         return dict(zip(('vendor', 'model', 'serial', 'firmware'), idparts))
 
     def connect_message(self, idn_param: str = 'IDN',
-                        begin_time: float = None) -> None:
+                        begin_time: Optional[float] = None) -> None:
         """
         Print a standard message on initial connection to an instrument.
 
