@@ -1,5 +1,5 @@
 import logging
-from typing import Dict, Callable, List, Optional, Sequence
+from typing import Dict, Callable, Optional, Sequence, Any
 from functools import partial
 
 import numpy as np
@@ -8,6 +8,7 @@ from qcodes import VisaInstrument, validators as vals
 from qcodes import InstrumentChannel, ChannelList, Instrument
 from qcodes import ArrayParameter
 from qcodes.utils.validators import Enum, Numbers
+from qcodes.instrument.parameter import ParamRawDataType
 
 
 log = logging.getLogger(__name__)
@@ -26,7 +27,7 @@ class RawTrace(ArrayParameter):
     raw_trace will return a trace from OSCIL
     """
 
-    def __init__(self, name, instrument, channel):
+    def __init__(self, name: str, instrument: Instrument, channel: int):
         super().__init__(name,
                          shape=(1024,),
                          label='Voltage',
@@ -36,11 +37,11 @@ class RawTrace(ArrayParameter):
                              f'Channel {channel} time series',),
                          setpoint_units=('s',),
                          docstring='raw trace from the scope',
+                         instrument=instrument
                          )
         self._channel = channel
-        self._instrument = instrument
 
-    def prepare_curvedata(self):
+    def prepare_curvedata(self) -> None:
         """
         Prepare the scope for returning curve data
         """
@@ -50,6 +51,8 @@ class RawTrace(ArrayParameter):
 
         # shorthand
         instr = self._instrument
+        assert isinstance(instr, InfiniiumChannel)
+
         # number of set points
         self.npts = int(instr.ask("WAV:POIN?"))
         # first set point
@@ -65,14 +68,18 @@ class RawTrace(ArrayParameter):
         self.shape = (self.npts, )
 
         # make this on a per channel basis?
-        self._instrument._parent.trace_ready = True
+        root_instrument = instr.root_instrument
+        assert isinstance(root_instrument, Infiniium)
+        root_instrument.trace_ready = True
 
-    def get_raw(self):
+    def get_raw(self) -> ParamRawDataType:
         # when get is called the setpoints have to be known already
         # (saving data issue). Therefor create additional prepare function that
         # queries for the size.
         # check if already prepared
-        if not self._instrument._parent.trace_ready:
+        assert isinstance(self._instrument, InfiniiumChannel)
+
+        if not self._instrument.root_instrument.trace_ready:
             raise TraceNotReady('Please run prepare_curvedata to prepare '
                                 'the scope for acquiring a trace.')
 
@@ -159,7 +166,7 @@ class MeasurementSubsystem(InstrumentChannel):
     # note: this is not really a channel, but InstrumentChannel does everything
     # a 'Submodule' class should do
 
-    def __init__(self, parent: Instrument, name: str, **kwargs) -> None:
+    def __init__(self, parent: Instrument, name: str, **kwargs: Any) -> None:
         super().__init__(parent, name, **kwargs)
 
         self.add_parameter(name='source_1',
@@ -295,7 +302,7 @@ class MeasurementSubsystem(InstrumentChannel):
 
 class InfiniiumChannel(InstrumentChannel):
 
-    def __init__(self, parent, name, channel):
+    def __init__(self, parent: "Infiniium", name: str, channel: int):
         super().__init__(parent, name)
         # display
         self.add_parameter(name='display',
@@ -357,14 +364,16 @@ class Infiniium(VisaInstrument):
      - tested for MSOS104A of the Infiniium S-series.
     """
 
-    def __init__(self, name, address, timeout=20, **kwargs):
+    def __init__(self, name: str, address: str,
+                 timeout: float = 20,
+                 **kwargs: Any):
         """
         Initialises the oscilloscope.
 
         Args:
-            name (str): Name of the instrument used by QCoDeS
-        address (string): Instrument address as used by VISA
-            timeout (float): visa timeout, in secs.
+            name: Name of the instrument used by QCoDeS
+            address: Instrument address as used by VISA
+            timeout: visa timeout, in secs.
         """
 
         super().__init__(name, address, timeout=timeout,
@@ -538,7 +547,7 @@ class Infiniium(VisaInstrument):
                            )
         # Channels
         channels = ChannelList(self, "Channels", InfiniiumChannel,
-                                snapshotable=False)
+                               snapshotable=False)
 
         for i in range(1,5):
             channel = InfiniiumChannel(self, f'chan{i}', i)
@@ -554,7 +563,7 @@ class Infiniium(VisaInstrument):
     def _cmd_and_invalidate(self, cmd: str) -> Callable:
         return partial(Infiniium._cmd_and_invalidate_call, self, cmd)
 
-    def _cmd_and_invalidate_call(self, cmd: str, val) -> None:
+    def _cmd_and_invalidate_call(self, cmd: str, val: float) -> None:
         """
         executes command and sets trace_ready status to false
         any command that effects the number of setpoints should invalidate the trace
@@ -562,7 +571,7 @@ class Infiniium(VisaInstrument):
         self.trace_ready = False
         self.write(cmd.format(val))
 
-    def save_data(self, filename):
+    def save_data(self, filename: str) -> None:
         """
         Saves the channels currently shown on oscilloscope screen to a USB.
         Must set data_format parameter prior to calling this
