@@ -1,15 +1,17 @@
+from typing import Dict, Optional
+
 from qcodes import IPInstrument
 from qcodes.utils import validators as vals
 from qcodes.instrument.channel import InstrumentChannel, ChannelList
 
 
 class MC_channel(InstrumentChannel):
-    def __init__(self, parent, name, channel_letter):
+    def __init__(self, parent: "RC_SPDT", name: str, channel_letter: str):
         """
         Args:
-            parent (Instrument): The instrument the channel is a part of
-            name (str): the name of the channel
-            channel_letter (str): channel letter ['a', 'b', 'c' or 'd'])
+            parent: The instrument the channel is a part of
+            name: the name of the channel
+            channel_letter: channel letter ['a', 'b', 'c' or 'd'])
         """
 
         super().__init__(parent, name)
@@ -24,10 +26,10 @@ class MC_channel(InstrumentChannel):
                            vals=vals.Ints(1, 2)
                            )
 
-    def _set_switch(self, switch):
+    def _set_switch(self, switch: int) -> None:
         self.write('SET{}={}'.format(self.channel_letter, switch-1))
 
-    def _get_switch(self):
+    def _get_switch(self) -> int:
         val = int(self.ask('SWPORT?'))
         # select out bit in return number
         # corisponding to channel switch configuration
@@ -41,11 +43,11 @@ class RC_SPDT(IPInstrument):
     Mini-Circuits SPDT RF switch
 
     Args:
-            name (str): the name of the instrument
-            address (str): ip address ie "10.0.0.1"
-            port (int): port to connect to default Telnet:23
+        name: the name of the instrument
+        address: ip address ie "10.0.0.1"
+        port: port to connect to default Telnet:23
     """
-    def __init__(self, name, address, port=23):
+    def __init__(self, name: str, address: str, port: int = 23):
         super().__init__(name, address, port)
         self.flush_connection()
 
@@ -65,19 +67,21 @@ class RC_SPDT(IPInstrument):
 
         self.connect_message()
 
-    def ask(self, cmd):
+    def ask(self, cmd: str) -> str:
         ret = self.ask_raw(cmd)
         ret = ret.strip()
         return ret
 
-    def get_idn(self):
+    def get_idn(self) -> Dict[str, Optional[str]]:
 
         fw = self.ask('FIRMWARE?')
         MN = self.ask('MN?')
         SN = self.ask('SN?')
 
-        id_dict = {'firmware': fw,
-                   'model': MN[3:],
-                   'serial': SN[3:],
-                   'vendor': 'Mini-Circuits'}
+        id_dict: Dict[str, Optional[str]] = {
+            'firmware': fw,
+            'model': MN[3:],
+            'serial': SN[3:],
+            'vendor': 'Mini-Circuits'
+        }
         return id_dict
