@@ -27,12 +27,12 @@ def chain(*functions: Callable) -> Callable:
         >>> chain(f, float)()  # return 1.2 as float
     """
 
-    def make_iter(args):
+    def make_iter(args: Any) -> Iterable:
         if not isinstance(args, Iterable) or isinstance(args, str):
             return args,
         return args
 
-    def inner(*args):
+    def inner(*args: Any) -> Any:
         result = args
         for fun in functions:
             new_args = make_iter(result)
@@ -144,7 +144,7 @@ class Stahl(VisaInstrument):
         address: A serial port address
     """
 
-    def __init__(self, name: str, address: str, **kwargs):
+    def __init__(self, name: str, address: str, **kwargs: Any):
         super().__init__(name, address, terminator="\r", **kwargs)
         assert isinstance(self.visa_handle, SerialInstrument)
 
@@ -197,7 +197,7 @@ class Stahl(VisaInstrument):
         return response
 
     @staticmethod
-    def parse_idn_string(idn_string) -> Dict[str, Any]:
+    def parse_idn_string(idn_string: str) -> Dict[str, Any]:
         """
         Return:
              dict: The dict contains the following keys "model",
@@ -248,5 +248,5 @@ class Stahl(VisaInstrument):
         }
 
     @property
-    def identifier(self):
+    def identifier(self) -> str:
         return f"{self.model}{self.serial_number}"
