@@ -118,7 +118,7 @@ def test_do0d_with_a_callable(_param_callable, period, plot):
 @pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
 @pytest.mark.parametrize('period, plot', [(None, True), (None, False),
                          (1, True), (1, False)])
-def test_do0d_with_multiparameters(_param, _param_complex, period, plot):
+def test_do0d_with_2_parameters(_param, _param_complex, period, plot):
     do0d(_param, _param_complex, write_period=period, do_plot=plot)
 
 
@@ -172,8 +172,6 @@ def test_do0d_verify_shape(_param, _param_complex, multiparamtype,
     dummyinstrument.A.dummy_stop(1)
     dummyinstrument.A.dummy_n_points(n_points_pws)
 
-
-
     results = do0d(arrayparam, multiparam, paramwsetpoints,
                    _param, _param_complex,
                    do_plot=False)
@@ -186,6 +184,16 @@ def test_do0d_verify_shape(_param, _param_complex, multiparamtype,
     expected_shapes[paramwsetpoints.full_name] = (n_points_pws, )
     assert results[0].description.shapes == expected_shapes
 
+    ds = results[0]
+
+    assert ds.description.shapes == expected_shapes
+
+    data = ds.get_parameter_data()
+
+    for name, data in data.items():
+        for param_data in data.values():
+            assert param_data.shape == expected_shapes[name]
+
 
 @pytest.mark.usefixtures("temp_exp", "temp_db")
 def test_do0d_parameter_with_array_vals():
@@ -219,7 +227,7 @@ def test_do1d_with_complex_parameter(_param_set, _param_complex, delay):
 
 @pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
 @pytest.mark.parametrize('delay', [0, 0.1, 1])
-def test_do1d_with_multiparameter(_param_set, _param, _param_complex, delay):
+def test_do1d_with_2_parameter(_param_set, _param, _param_complex, delay):
 
     start = 0
     stop = 1
@@ -303,7 +311,16 @@ def test_do1d_parameter_with_array_vals(_param_set):
     results = do1d(_param_set, start, stop, num_points, delay,
                    param, do_plot=False)
     expected_shapes = {'paramwitharrayval': (num_points, 10)}
-    assert results[0].description.shapes == expected_shapes
+
+    ds = results[0]
+
+    assert ds.description.shapes == expected_shapes
+
+    data = ds.get_parameter_data()
+
+    for name, data in data.items():
+        for param_data in data.values():
+            assert param_data.shape == expected_shapes[name]
 
 
 @pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
@@ -366,24 +383,31 @@ def test_do2d_output_data(_param, _param_complex, _param_set, _param_set_2):
     assert data.parameters == (f'{_param_set.name},{_param_set_2.name},'
                                f'{_param.name},{_param_complex.name}')
     loaded_data = data.get_parameter_data()
+    expected_data_1 = np.ones(25).reshape(num_points_p1, num_points_p2)
+
     np.testing.assert_array_equal(loaded_data[_param.name][_param.name],
-                                  np.ones(25))
-    np.testing.assert_array_equal(loaded_data[_param_complex.name][_param_complex.name],
-                                  (1+1j)*np.ones(25))
+                                  expected_data_1)
+    expected_data_2 = (1+1j)*np.ones(25).reshape(num_points_p1, num_points_p2)
+    np.testing.assert_array_equal(
+        loaded_data[_param_complex.name][_param_complex.name],
+        expected_data_2
+    )
 
-    expected_setpoints_1 = np.repeat(np.linspace(start_p1,
-                                                 stop_p1,
-                                                 num_points_p1),
-                                     num_points_p2)
-    np.testing.assert_array_equal(loaded_data[_param_complex.name][_param_set.name],
-                                  expected_setpoints_1)
+    expected_setpoints_1 = np.repeat(
+        np.linspace(start_p1, stop_p1, num_points_p1),
+        num_points_p2).reshape(num_points_p1, num_points_p2)
+    np.testing.assert_array_equal(
+        loaded_data[_param_complex.name][_param_set.name],
+        expected_setpoints_1
+    )
 
-    expected_setpoints_2 = np.tile(np.linspace(start_p2,
-                                               stop_p2,
-                                               num_points_p2),
-                                   num_points_p1)
-    np.testing.assert_array_equal(loaded_data[_param_complex.name][_param_set_2.name],
-                                  expected_setpoints_2)
+    expected_setpoints_2 = np.tile(
+        np.linspace(start_p2, stop_p2, num_points_p2),
+                    num_points_p1).reshape(num_points_p1, num_points_p2)
+    np.testing.assert_array_equal(
+        loaded_data[_param_complex.name][_param_set_2.name],
+        expected_setpoints_2
+    )
 
 
 @pytest.mark.usefixtures("temp_exp", "temp_db")
@@ -435,6 +459,13 @@ def test_do2d_verify_shape(_param, _param_complex, _param_set, _param_set_2,
                                                   n_points_pws)
 
     assert results[0].description.shapes == expected_shapes
+    ds = results[0]
+
+    data = ds.get_parameter_data()
+
+    for name, data in data.items():
+        for param_data in data.values():
+            assert param_data.shape == expected_shapes[name]
 
 
 @pytest.mark.usefixtures("plot_close", "temp_exp", "temp_db")
@@ -527,6 +558,7 @@ def test_do2d_additional_setpoints(_param, _param_complex,
 
 @given(num_points_p1=hst.integers(min_value=1, max_value=10),
        num_points_p2=hst.integers(min_value=1, max_value=10))
+@settings(deadline=None)
 @pytest.mark.usefixtures("temp_exp", "temp_db")
 def test_do2d_additional_setpoints_shape(
         _param, _param_complex,
