@@ -1,28 +1,26 @@
+import collections
 import io
 import json
 import logging
 import math
 import numbers
-import time
 import os
-from pathlib import Path
-import collections
-
-from collections.abc import Iterator, Sequence, Mapping
-from copy import deepcopy
-from typing import (Dict, Any, Type, List, Tuple, Union, Optional,
-                    cast, Callable, SupportsAbs)
-from typing import Sequence as TSequence
-from contextlib import contextmanager
+import time
 from asyncio import iscoroutinefunction
-from inspect import signature
+from collections import OrderedDict, abc
+from contextlib import contextmanager
+from copy import deepcopy
 from functools import partial
-from collections import OrderedDict
+from inspect import signature
+from pathlib import Path
+from typing import (TYPE_CHECKING, Any, Callable, Dict, Iterator, List,
+                    Mapping, MutableMapping, Optional, Sequence, SupportsAbs,
+                    Tuple, Type, Union, cast)
 
 import numpy as np
 
-from qcodes.utils.deprecate import deprecate
-
+if TYPE_CHECKING:
+    from PyQt5.QtWidgets import QMainWindow
 
 QCODES_USER_PATH_ENV = 'QCODES_USER_PATH'
 
@@ -40,7 +38,7 @@ class NumpyJSONEncoder(json.JSONEncoder):
     ``default`` method for the description of all conversions.
     """
 
-    def default(self, obj):
+    def default(self, obj: Any) -> Any:
         """
         List of conversions that this encoder performs:
         * ``numpy.generic`` (all integer, floating, and other types) gets
@@ -76,7 +74,8 @@ class NumpyJSONEncoder(json.JSONEncoder):
             }
         elif hasattr(obj, '_JSONEncoder'):
             # Use object's custom JSON encoder
-            return obj._JSONEncoder()
+            jsosencode = getattr(obj, "_JSONEncoder")
+            return jsosencode()
         else:
             try:
                 s = super().default(obj)
@@ -91,7 +90,7 @@ class NumpyJSONEncoder(json.JSONEncoder):
                 if hasattr(obj, '__getnewargs__'):
                     return {
                         '__class__': type(obj).__name__,
-                        '__args__': obj.__getnewargs__()
+                        '__args__': getattr(obj, "__getnewargs__")()
                     }
                 else:
                     # we cannot convert the object to JSON, just take a string
@@ -99,7 +98,7 @@ class NumpyJSONEncoder(json.JSONEncoder):
             return s
 
 
-def tprint(string, dt=1, tag='default'):
+def tprint(string: str, dt: int = 1, tag: str = 'default') -> None:
     """Print progress of a loop every ``dt`` seconds."""
     ptime = _tprint_times.get(tag, 0)
     if (time.time() - ptime) > dt:
@@ -107,14 +106,14 @@ def tprint(string, dt=1, tag='default'):
         _tprint_times[tag] = time.time()
 
 
-def is_sequence(obj):
+def is_sequence(obj: Any) -> bool:
     """
     Test if an object is a sequence.
 
     We do not consider strings or unordered collections like sets to be
     sequences, but we do accept iterators (such as generators).
     """
-    return (isinstance(obj, (Iterator, Sequence, np.ndarray)) and
+    return (isinstance(obj, (abc.Iterator, abc.Sequence, np.ndarray)) and
             not isinstance(obj, (str, bytes, io.IOBase)))
 
 
@@ -122,7 +121,7 @@ def is_sequence_of(obj: Any,
                    types: Optional[Union[Type[object],
                                          Tuple[Type[object], ...]]] = None,
                    depth: Optional[int] = None,
-                   shape: Optional[TSequence[int]] = None
+                   shape: Optional[Sequence[int]] = None
                    ) -> bool:
     """
     Test if object is a sequence of entirely certain class(es).
@@ -167,7 +166,7 @@ def is_sequence_of(obj: Any,
     return True
 
 
-def is_function(f: Callable, arg_count: int, coroutine: bool=False) -> bool:
+def is_function(f: Callable, arg_count: int, coroutine: bool = False) -> bool:
     """
     Check and require a function that can accept the specified number of
     positional arguments, which either is or is not a coroutine
@@ -209,12 +208,12 @@ def is_function(f: Callable, arg_count: int, coroutine: bool=False) -> bool:
         return False
 
 
-def full_class(obj):
+def full_class(obj: object) -> str:
     """The full importable path to an object's class."""
     return type(obj).__module__ + '.' + type(obj).__name__
 
 
-def named_repr(obj):
+def named_repr(obj: Any) -> str:
     """Enhance the standard repr() with the object's name attribute."""
     s = '<{}.{}: {} at {}>'.format(
         obj.__module__,
@@ -224,7 +223,7 @@ def named_repr(obj):
     return s
 
 
-def deep_update(dest, update):
+def deep_update(dest: MutableMapping, update: Mapping) -> MutableMapping:
     """
     Recursively update one JSON structure with another.
 
@@ -234,7 +233,7 @@ def deep_update(dest, update):
     """
     for k, v_update in update.items():
         v_dest = dest.get(k)
-        if isinstance(v_update, Mapping) and isinstance(v_dest, Mapping):
+        if isinstance(v_update, abc.Mapping) and isinstance(v_dest, abc.MutableMapping):
             deep_update(v_dest, v_update)
         else:
             dest[k] = deepcopy(v_update)
@@ -319,7 +318,7 @@ def make_sweep(start: float,
     return cast(List[float], output_list)
 
 
-def wait_secs(finish_clock):
+def wait_secs(finish_clock: float) -> float:
     """
     Calculate the number of seconds until a given clock time.
     The clock time should be the result of ``time.perf_counter()``.
@@ -363,7 +362,7 @@ class DelegateAttributes:
     to *not* delegate to any other dictionary or object.
     """
 
-    def __getattr__(self, key):
+    def __getattr__(self, key: str) -> Any:
         if key in self.omit_delegate_attrs:
             raise AttributeError("'{}' does not delegate attribute {}".format(
                 self.__class__.__name__, key))
@@ -397,8 +396,8 @@ class DelegateAttributes:
             "'{}' object and its delegates have no attribute '{}'".format(
                 self.__class__.__name__, key))
 
-    def __dir__(self):
-        names = super().__dir__()
+    def __dir__(self) -> List[str]:
+        names = list(super().__dir__())
         for name in self.delegate_attr_dicts:
             d = getattr(self, name, None)
             if d is not None:
@@ -414,14 +413,14 @@ class DelegateAttributes:
         return sorted(set(names))
 
 
-def strip_attrs(obj, whitelist=()):
+def strip_attrs(obj: object, whitelist: Sequence[str] = ()) -> None:
     """
     Irreversibly remove all direct instance attributes of object, to help with
     disposal, breaking circular references.
 
     Args:
         obj: Object to be stripped.
-        whitelist (list): List of names that are not stripped from the object.
+        whitelist: List of names that are not stripped from the object.
     """
     try:
         lst = set(list(obj.__dict__.keys())) - set(whitelist)
@@ -437,9 +436,9 @@ def strip_attrs(obj, whitelist=()):
 
 
 def compare_dictionaries(dict_1: Dict, dict_2: Dict,
-                         dict_1_name: Optional[str]='d1',
-                         dict_2_name: Optional[str]='d2',
-                         path: str="") -> Tuple[bool, str]:
+                         dict_1_name: Optional[str] = 'd1',
+                         dict_2_name: Optional[str] = 'd2',
+                         path: str = "") -> Tuple[bool, str]:
     """
     Compare two dictionaries recursively to find non matching elements.
 
@@ -502,12 +501,12 @@ def compare_dictionaries(dict_1: Dict, dict_2: Dict,
     return dicts_equal, dict_differences
 
 
-def warn_units(class_name, instance):
+def warn_units(class_name: str, instance: object) -> None:
     logging.warning('`units` is deprecated for the `' + class_name +
                     '` class, use `unit` instead. ' + repr(instance))
 
 
-def foreground_qt_window(window):
+def foreground_qt_window(window: "QMainWindow") -> None:
     """
     Try as hard as possible to bring a qt window to the front. This
     will use pywin32 if installed and running on windows as this
@@ -522,8 +521,9 @@ def foreground_qt_window(window):
         >>> Qtplot.qt_helpers.foreground_qt_window(plot.win)
     """
     try:
-        from win32gui import SetWindowPos
         import win32con
+        from win32gui import SetWindowPos
+
         # use the idea from
         # https://stackoverflow.com/questions/12118939/how-to-make-a-pyqt4-window-jump-to-the-front
         SetWindowPos(window.winId(),
@@ -541,7 +541,7 @@ def foreground_qt_window(window):
     window.activateWindow()
 
 
-def add_to_spyder_UMR_excludelist(modulename: str):
+def add_to_spyder_UMR_excludelist(modulename: str) -> None:
     """
     Spyder tries to reload any user module. This does not work well for
     qcodes because it overwrites Class variables. QCoDeS uses these to
@@ -564,7 +564,8 @@ def add_to_spyder_UMR_excludelist(modulename: str):
             sitecustomize_found = True
         if sitecustomize_found is False:
             try:
-                from spyder_kernels.customize import spydercustomize as sitecustomize
+                from spyder_kernels.customize import \
+                    spydercustomize as sitecustomize
 
             except ImportError:
                 pass
@@ -585,7 +586,9 @@ def add_to_spyder_UMR_excludelist(modulename: str):
 
 
 @contextmanager
-def attribute_set_to(object_: Any, attribute_name: str, new_value: Any):
+def attribute_set_to(object_: object,
+                     attribute_name: str,
+                     new_value: Any) -> Iterator[None]:
     """
     This context manager allows to change a given attribute of a given object
     to a new value, and the original value is reverted upon exit of the context
@@ -605,7 +608,9 @@ def attribute_set_to(object_: Any, attribute_name: str, new_value: Any):
         setattr(object_, attribute_name, old_value)
 
 
-def partial_with_docstring(func: Callable, docstring: str, **kwargs):
+def partial_with_docstring(func: Callable,
+                           docstring: str,
+                           **kwargs: Any) -> Callable:
     """
     We want to have a partial function which will allow us access the docstring
     through the python built-in help function. This is particularly important
@@ -626,7 +631,7 @@ def partial_with_docstring(func: Callable, docstring: str, **kwargs):
     """
     ex = partial(func, **kwargs)
 
-    def inner(**inner_kwargs):
+    def inner(**inner_kwargs: Any) -> None:
         ex(**inner_kwargs)
 
     inner.__doc__ = docstring
@@ -676,7 +681,7 @@ def abstractmethod(funcobj: Callable) -> Callable:
     return funcobj
 
 
-def _ruamel_importer():
+def _ruamel_importer() -> type:
     try:
         from ruamel_yaml import YAML
     except ImportError:
